/*
  mainwindow.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG

#include "mainwindow.h"

#include <qwidget.h>
#include <stdio.h>
#include "qgmap.h"
#include "qgconf.h"
#include "version.h"

#include <qdialog.h>
#include <qdatetime.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qvbox.h>
#include <qlayout.h>

#include "pos.h"

#include "mounter.h"

#include "mappathdialog.h"
#include "gammadialog.h"
#include "poi_dialog.h"
#include "poi_search_dialog.h"

#ifndef QWS
#include <qlabel.h>
#endif
#ifdef QWS
#include <qcopchannel_qws.h>
#endif

#include "mainmenu.h"
#include "gpsmenu.h"

MainWindow::MainWindow( QWidget *parent, const char *name, WFlags fl) :
	QWidget(parent, name, fl)
{
	conf = new QGConf(this, "config");

#ifndef QWS
	resize(640,640);
#endif
	setCaption(QString::fromUtf8("qgmap - ") + QGMAP_VER);

	QVBoxLayout *vbox = new QVBoxLayout(this);
	menubar = new MainMenu(this, "menubar");

	connect(menubar, SIGNAL(activated(int)),
			this, SLOT(menu_activated(int)));

	main_widget = new MainWidget(this, "mainwidget");

	vbox->addWidget(menubar);
	vbox->addWidget(main_widget);

	mounter = new Mounter(this, "mounter");

	/* connect signal */
	connect(conf, SIGNAL(configChanged(const QString &, const QString &)),
			main_widget, SIGNAL(configChanged(const QString &, const QString &)));
	connect(this, SIGNAL(setPosition(Pos &)),
			main_widget, SIGNAL(setPosition(Pos &)));
	connect(this, SIGNAL(reload(bool)),
			main_widget, SIGNAL(reload(bool)));
	connect(this, SIGNAL(cacheFlush()),
			main_widget, SIGNAL(cacheFlush()));

#ifndef QWS
	label = new QLabel(this);
	label->setGeometry(640-16,640-16,16,16);
#endif

#ifdef QWS
	QCopChannel *qcop = new QCopChannel(QCOPNAME, this);
	connect(qcop, SIGNAL(received(const QCString &, const QByteArray&)),
			this, SLOT(recvQCop(const QCString &, const QByteArray&)));
#endif
	conf->migrate();

	setFocusProxy(main_widget);
}

MainWindow::~MainWindow()
{
}

void MainWindow::showEvent( QShowEvent *e )
{
	UNUSED(e);

	conf->reloadEntry("GPSDev", DEFAULT_GPS_DEVICE);
	conf->reloadEntry("PreCommand", DEFAULT_PRE_COMMAND);
	conf->reloadEntry("PostCommand", DEFAULT_POST_COMMAND);
	conf->reloadEntry("GPSInterval", STR(DEFAULT_GPS_INTERVAL));
	conf->reloadEntry("GPSDatum", STR((int)DEFAULT_GPS_DATUM));
	conf->reloadEntry("GPSTimeZone", STR(DEFAULT_GPS_TIMEZONE));

	main_widget->sendRequest("menu", QString::number(EV_JUMP_MARKER_0));
}

void MainWindow::recvQCop(const QCString &m, const QByteArray &d)
{
	if(m == "setWGSLocation(QString)"){
		QString str;
		QDataStream in(d, IO_ReadOnly);
		in >> str;

		Pos pos(main_widget->getPosition());
		pos.setPos(str, true);

		emit setPosition(pos);
	} else if(m == "setTokyoLocation(QString)"){
		QString str;
		QDataStream in(d, IO_ReadOnly);
		in >> str;

		Pos pos(main_widget->getPosition());
		pos.setPos(str, false);

		emit setPosition(pos);
	}
}

void MainWindow::menu_activated(int id)
{
	int ret;

	switch (id) {
	case EV_SET_MAP_PATH:
	{
		MapPathDialog dlg(main_widget, "MapPath");
		dlg.loadConfig();
		ret = dlg.exec();
		if(ret == QDialog::Accepted){
			dlg.saveConfig();
			emit cacheFlush();
			emit reload(true);
		}
		break;
	}
	case EV_SET_GAMMA:
	{
		GammaDialog dlg(main_widget, "Gamma", TRUE);

		QString gamma = conf->readEntry("Gamma", DEFAULT_GAMMA);
		dlg.setDefault(gamma.toDouble());

		ret = dlg.exec();
		if(ret == QDialog::Accepted){
			conf->writeEntry("Gamma", QString::number(dlg.getGamma()));
		}
		break;
	}
	case EV_POI_MANAGE:
	{
		PoiDialog dlg(main_widget, "Poi", TRUE);

		ret = dlg.exec();
		emit cacheFlush();
		emit reload(true);
		break;
	}
	case EV_POI_SEARCH:
	{
		PoiSearchDialog dlg(main_widget, main_widget->getPosition());

		ret = dlg.exec();
		printf("ret = %d\n", ret);
		if (ret == QDialog::Accepted) {
			Pos pos = dlg.getPos();
			printf("pos=%s\n", (const char *)pos.serialize());
			emit setPosition(pos);
		}
		break;
	}
	default:
		main_widget->sendRequest("menu", QString::number(id));
		break;
	}
}

void MainWindow::keyPressEvent(QKeyEvent * e)
{
	switch(e->key()){
	case Key_Menu:
	case Key_F11:	/* Zaurus Menu Key */
		menubar->setFocus();
		return;
	case Key_S:
		menu_activated(EV_POI_SEARCH);
		return;
	}

	main_widget->setFocus();
	main_widget->keyPressEvent(e);
}

void MainWindow::closeEvent(QCloseEvent *e)
{
	if (menubar->hasFocus()) {
		e->ignore();
		main_widget->setFocus();
	} else {
		e->accept();
	}
}

void MainWindow::disableMenu()
{
	menubar->setDisabled(true);
}

void MainWindow::enableMenu()
{
	menubar->setEnabled(true);
}
